@@ -192,15 +192,20 @@ GameArea::GameArea()
    focusBackgroundColor(0.8,0.8,0.8,Lum::OS::display->GetColor(Lum::OS::Display::fillColor)),
    errorColor(1.0,0,0,Lum::OS::display->GetColor(Lum::OS::Display::fillColor)),
    correctColor(0,0.5,0,Lum::OS::display->GetColor(Lum::OS::Display::blackColor)),
+   hintColor(1,1,0,Lum::OS::display->GetColor(Lum::OS::Display::blackColor)),
    boxSize(0),
    focusedCell(0),
    focusChangedAction(new Lum::Model::Action()),
+   hintMode(new Lum::Model::Boolean()),
    finished(new Lum::Model::Boolean())
 {
   SetCanFocus(true);
   RequestFocus();
 
+  hintMode->Set(false);
   finished->Set(true);
+
+  Observe(hintMode);
 }
 
 int GameArea::GetCellOffset(size_t index) const
@@ -208,6 +213,19 @@ int GameArea::GetCellOffset(size_t index) const
   return (index+1)*1+(index/3+1)*1+index*boxSize;
 }
 
+void GameArea::SetFinished(bool finished)
+{
+  this->finished->Set(finished);
+
+    hintMode->Set(false);
+    if (finished) {
+      hintMode->Disable();
+    }
+    else {
+      hintMode->Enable();
+    }
+}
+
 void GameArea::SetModel(const Sudoku &riddle, const Sudoku &solution)
 {
   this->solution=solution;
@@ -218,7 +236,7 @@ void GameArea::SetModel(const Sudoku &riddle, const Sudoku &solution)
     marks[i]=0;
   }
 
-  finished->Set(game==solution);
+  SetFinished(game==solution);
   focusedCell=0;
 
   Redraw();
@@ -243,7 +261,7 @@ void GameArea::SetGame(const Game& game)
     }
   }
 
-  finished->Set(this->game==solution);
+  SetFinished(this->game==solution);
 
   focusedCell=0;
   Redraw();
@@ -494,6 +512,20 @@ void GameArea::Draw(int x, int y, size_t w, size_t h)
                        text);
       draw->PopForeground();
     }
+    else if (!finished->Get() &&
+             hintMode->Get() &&
+             i==focusedCell) {
+      std::wstring        text(Lum::Base::NumberToWString(solution.Get(i)));
+      Lum::OS::FontExtent extent;
+
+      font->StringExtent(text,extent,Lum::OS::Font::bold);
+
+      draw->PushForeground(hintColor);
+      draw->DrawString(this->x+GetCellOffset(i%Sudoku::dimension)+(boxSize-extent.width+extent.left+extent.right)/2-extent.left,
+                       this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-extent.height)/2+extent.ascent,
+                       text);
+      draw->PopForeground();
+    }
     else {
       for (size_t j=0; j<=8; j++) {
         if (marks[i] & (1 << j)) {
@@ -600,7 +632,7 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
         if (selector->HasSelection()) {
           game.Set(focusedCell,selector->GetValue());
           marks[focusedCell]=0;
-          finished->Set(game==solution);
+          SetFinished(game==solution);
           Redraw();
         }
 
@@ -668,7 +700,7 @@ bool GameArea::HandleKeyEvent(const Lum::OS::KeyEvent& event)
         if (value>=0 && value<=Sudoku::dimension) {
           if (riddle.Get(focusedCell)==Sudoku::empty) {
             game.Set(focusedCell,value);
-            finished->Set(game==solution);
+            SetFinished(game==solution);
             Redraw();
             // Focus did not changed, but we simulate it to get the value change
             // notified
@@ -690,6 +722,11 @@ Lum::Model::Action* GameArea::GetFocusChangedAction() const
   return focusChangedAction.Get();
 }
 
+Lum::Model::Boolean* GameArea::GetHintMode() const
+{
+  return hintMode.Get();
+}
+
 Lum::Model::Boolean* GameArea::GetFinishedModel() const
 {
   return finished.Get();
@@ -743,12 +780,21 @@ void GameArea::SetFocusedCellValue(size_t value)
   Redraw();
 
   focusChangedAction->Trigger();
-  finished->Set(game==solution);
+  SetFinished(game==solution);
 }
 
 void GameArea::Solve()
 {
-  finished->Set(true);
+  SetFinished(true);
   Redraw();
 }
 
+void GameArea::Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
+{
+  if (model==hintMode) {
+    Redraw();
+  }
+
+  Object::Resync(model,msg);
+}
+
