@@ -249,7 +249,7 @@ public:
       array->Add(toggle);
     }
     else {
-      array->Add(Lum::Toggle::Create(_(L"GAME_Hint",L"_Hint"),gameArea->GetHintMode(),true,false));
+      array->Add(Lum::Toggle::Create(_(L"GAME_Hint",L"Hint"),gameArea->GetHintMode(),true,false));
     }
     if (statisticsImage->IsValid()) {
       array->Add(Lum::Button::Create(Lum::Image::Create(statisticsImage),statAction,true,false));
